#
# Authors: Brian Skahan <bskahan@etria.com>
#          Tom von Schwerdtner <tvon@etria.com>
#
# Copyright 2004-2005, Etria, LLP
#
# This file is part of Quills
#
# Quills is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Quills is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Quills; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##############################################################################
"""Migrate from 0.8 to 0.9."""

from Products.Archetypes.BaseBTreeFolder import BaseBTreeFolder
from Products.CMFCore.utils import getToolByName
from Products.Quills import MetaWeblogAPI, BloggerAPI


class Migration(object):
    """Migrate from 0.8 to 0.9."""

    def __init__(self, site):
        self.site = site

    def migrate(self):
        """Run migration on site object passed to __init__."""
        print u"Migrating Quills 0.8 -> 0.9"

        #XXX First migrate QuillsTool

        self.catalog = getToolByName(self.site, 'portal_catalog')
        self.qtool = getToolByName(self.site, 'quills_tool')

        weblogs = self.catalog(meta_type='Weblog')
        for weblog in weblogs:
            self.migrateWeblog(weblog.getObject())

    def migrateWeblog(self, weblog):
        """Migrate a weblog."""
        print u'Migrating Weblog: %s' % weblog.Title()

        # I think we could do something like this:
        # entries = BaseBTreeFolder(weblog).getFolderListingFolderContents()
        # Since objects are stored differently in BTree's... not sure.
        print entries

        delattr(weblog, 'metaWeblog')
        delattr(weblog, 'blogger')

        RPCAuth = self.qtool.findRPCAuth(weblog)

        weblog.metaWeblog = MetaWeblogAPI.MetaWeblogAPI().__of__(weblog)
        weblog.metaWeblog.setupRPCAuth(RPCAuth)

        weblog.blogger = BloggerAPI.BloggerAPI().__of__(weblog)
        weblog.blogger.setupRPCAuth(RPCAuth)

        if not hasattr(weblog.aq_inner.aq_explicit, 'archive'):
            self.site.portal_types.constructContent('WeblogArchive',
                                               weblog,
                                               'archive',
                                               title='Archive',
                                               archive_type='root')

        if not hasattr(weblog.aq_inner.aq_explicit, 'drafts'):
            self.site.portal_types.constructContent('WeblogDrafts',
                                               weblog,
                                               'drafts',
                                               title='Drafts')



    def migrateWeblogEntry(self, entry):
        """Migrate a weblog entry."""

        # XXX 
        #  1- Move from folderish based object
        #  2- update schema
        #  3- ..

    def migrateWeblogTopic(self, topic):
        """Migrate a weblog topic."""


