#
# Authors: Brian Skahan <bskahan@etria.com>
#          Tom von Schwerdtner <tvon@etria.com>
#
# Copyright 2004-2005, Etria, LLP
#
# This file is part of Quills
#
# Quills is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Quills is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Quills; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##############################################################################
"""Migrate from 0.8 to 0.9."""

# CMF imports
from Products.CMFCore.utils import getToolByName

# Product imports
from Products.Quills import MetaWeblogAPI, BloggerAPI

# plonetrackback imports
from Products.plonetrackback.interfaces import ITrackbackInManager

# Standard library imports
from cStringIO import StringIO

class Migration(object):
    """Migrate from 0.9 to 1.0"""

    def __init__(self, site):
        self.site = site

    def migrate(self):
        """Run migration on site object passed to __init__."""
        print u"Migrating Quills 0.9 -> 1.0"

        #XXX First migrate QuillsTool

        self.catalog = getToolByName(self.site, 'portal_catalog')

        # XXX Need to remove the getEntryCategories index as it is now
        #     the same as the Subject index.

        weblogs = self.catalog(meta_type='Weblog')
        for weblog in weblogs:
            self.migrateWeblog(weblog.getObject())

    def migrateWeblog(self, weblog):
        """Migrate a weblog."""
        print u'Migrating Weblog: %s' % weblog.Title()

        # XXX:
        #  1- Move from BTree based object (?)
        #  2- Update schema (re-assign?)
        #  2- Re-create xml-rpc API's (need QuillsTool migrated)
        #  3- create archive folder
        #  4- loop entries and migrate

        self.migrateWeblogTopics(weblog)
        for brain in weblog.getAllEntries():
            entry = brain.getObject()
            self.migrateWeblogEntry(entry)

    def migrateWeblogEntry(self, entry):
        """Migrate a weblog entry."""
        self.migrateTrackbacksForEntry(entry)
        self.migrateCategoriesForEntry(entry)

    def migrateTrackbacksForEntry(self, entry):
        """Swap to multiple trackback URLs stored with an adapter.
        """
        # Adapt to the interface for storing trackback details
        tbim = ITrackbackInManager(entry)
        # Get the existing URLs by attribute access as the methods are no longer
        # available - the schema no longer defines them.
        urls = entry.trackbackURLs
        wftool = getToolByName(entry, 'portal_workflow')
        review_state = wftool.getInfoFor(entry, 'review_state')
        if review_state == 'published':
            # If the entry is published, the URLs will already have been pinged
            tbim.setPingedURLs(urls)
        else:
            # ... otherwise, they still need to be pinged.
            tbim.setURLsToPing(urls)

    def migrateCategoriesForEntry(self, entry):
        """
        """
        # Get the existing categories by direct attribute access as the methods
        # are no longer available - the schema no longer defines them.
        cats = list(entry.entryCategories)
        # Get the existing subject keywords
        subjects = list( entry.Subject() )
        # Add the two together
        subjects.extend(cats)
        # Set the new subjects list
        entry.setSubject(subjects)

    def migrateWeblogTopics(self, weblog):
        """Migrate a weblog topic."""
        # XXX Need to disable WeblogTopic content type after migration of topics
        if getattr(weblog.aq_explicit, 'topic_images', None) is None:
            weblog.invokeFactory('Folder', 'topic_images', title='Topic Images')
        topic_images = weblog.topic_images
        to_be_deleted = []
        for ob in weblog.objectValues():
            if getattr(ob, 'portal_type', None) == 'WeblogTopic':
                title = ob.Title()
                img = StringIO(ob.getTopicImage())
                if getattr(topic_images, title, None) is None:
                    topic_images.invokeFactory('Image', id=title)
                    newimg = getattr(topic_images, title)
                    newimg.setImage(img)
                to_be_deleted.append(ob.getId())
        weblog.manage_delObjects(ids=to_be_deleted)
