# Zope 3 imports
from zope.interface import implements

# Zope 2 imports
from Acquisition import Implicit

# basesyndication imports
from Products.basesyndication.interfaces import IFeedEntry, IFeedSource

# Archetypes imports
from Products.Archetypes.config import UUID_ATTR, REFERENCE_CATALOG
from Products.Archetypes.ExtensibleMetadata import FLOOR_DATE

# CMF imports
from Products.CMFCore.utils import getToolByName

# Product imports
from feed import FeedMixin

# basesyndication imports
from Products.basesyndication.interfaces import IFeed, IFeedSource


class BaseFeed(FeedMixin, Implicit):

    implements(IFeed)

    def __init__(self, context):
        self.context = context

    def getFeedSources(self):
        """See IFeed.
        For this implementation, we just adapt self.context to IFeedSource
        and return it as the only item in a sequence.
        """
        fs = IFeedSource(self.context)
        return [fs,]

    def getWebURL(self):
        """See IFeed.
        """
        return self.context.absolute_url()

    def getTitle(self):
        """See IFeed.
        """
        return self.context.Title()

    def getDescription(self):
        """See IFeed.
        """
        return self.context.Description()

    def getUID(self):
        """See IFeed.
        """
        return self.context.UID()


class BaseFeedSource:

    #implements(IFeedSource)

    def __init__(self, context):
        self.context = context


class BaseFeedEntry:
    """
    """

    #implements(IFeedEntry)

    def __init__(self, context):
        self.context = context

    def getWebURL(self):
        """See IFeedEntry.
        """
        return self.context.absolute_url()

    def getTitle(self):
        """See IFeedEntry.
        """
        return self.context.Title()

    def getDescription(self):
        """See IFeedEntry.
        """
        return self.context.Description()

    def getUID(self):
        """See IFeedEntry.
        """
        return self.context.UID()

    def getAuthor(self):
        """See IFeedEntry.
        """
        return self.context.Creator()

    def getEffectiveDate(self):
        """See IFeedEntry.
        """
        effective = self.context.effective()
        if effective == FLOOR_DATE:
            effective = self.context.created()
            # XXX But what if created returns the FLOOR_DATE too?
            # This will break feeds as python's datetime library needs dates above 1900.
        return effective

    def getModifiedDate(self):
        """See IFeedEntry.
        """
        return self.context.modified()

    def getTags(self):
        """See IFeedEntry.
        """
        return self.context.Subject()

    def getRights(self):
        """See IFeedEntry.
        """
        # XXX Implement me properly!
        return ''

    def getEnclosure(self):
        """See IFeedEntry.
        """
        # Override this method if you want to do podcasting.
        pass


class DocumentFeedEntry(BaseFeedEntry):
    """Adapter for CMFDefault's Document, which also works for talkback comment objects.
    """

    implements(IFeedEntry)

    def getUID(self):
        """See IFeedEntry.
        """
        uid = getattr(self.context, UUID_ATTR, None)
        if uid is None:
            refcat = getToolByName(self.context, REFERENCE_CATALOG)
            uid = refcat._getUUIDFor(self.context)
        return uid

    def getBody(self):
        """See IFeedEntry.
        """
        return self.context.CookedBody()

    def getEffectiveDate(self):
        """See IFeedEntry.
        """
        return self.context.modified()

