# Standard library imports
from types import ListType, TupleType

# Zope3 imports
from zope.interface import implements

# Zope2 imports
from AccessControl import ClassSecurityInfo

# CMF imports
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.public import BaseContent, registerType

# Product imports
from schema import referenced_feed_source_schema
from config import PROJECTNAME, REFERENCED_FEED_SOURCE_TYPE, REFERENCED_COMMENT_FEED_SOURCE_TYPE
import permissions as perms

# basesyndication imports
from Products.basesyndication.interfaces import IFeedSource, IFeedEntry


class ReferencedFeedSource(BaseContent):
    """An implementation of IFeedSource
    """

    implements(IFeedSource)

    security = ClassSecurityInfo()
    schema = referenced_feed_source_schema
    portal_type = meta_type = archetype_name = REFERENCED_FEED_SOURCE_TYPE
    global_allow = False

    def getFeedEntries(self):
        """See IFeedSource
        """
        source = self.getSource()
        return IFeedSource(source).getFeedEntries()


class ReferencedCommentFeedSource(BaseContent):
    """An implementation of IFeedSource that returns a sequence of the comments
    from each of the returned IFeedEntry instances for this source.
    """

    implements(IFeedSource)

    schema = referenced_feed_source_schema
    portal_type = meta_type = archetype_name = REFERENCED_COMMENT_FEED_SOURCE_TYPE
    global_allow = False

    def getFeedEntries(self):
        """See IFeedSource.
        """
        source = self.getSource()
        obs = IFeedSource(source).getFeedEntries()
        dtool = getToolByName(self, 'portal_discussion')
        feedentries = []
        for each in obs:
            replies = dtool.getDiscussionFor(each.context).getReplies()
            for reply in replies:
                feedentries.append( IFeedEntry(reply) )
        return feedentries


registerType(ReferencedFeedSource, PROJECTNAME)
registerType(ReferencedCommentFeedSource, PROJECTNAME)

