# Standard library imports
#from types import ListType, TupleType
import random

# Zope3 imports
from zope.interface import implements

# Zope2 imports
from AccessControl import ClassSecurityInfo

# CMF imports
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.public import BaseFolder, registerType

# Product imports
from schema import syndication_manager_schema
from config import PROJECTNAME, FEED_PORTAL_TYPE, SYNDICATION_MANAGER_PORTAL_TYPE
import permissions as perms

# basesyndication imports
from Products.basesyndication.interfaces import ISyndicationManager


class SyndicationManager(BaseFolder):
    """An implementation of ISyndicationManager
    """

    implements(ISyndicationManager)

    security = ClassSecurityInfo()
    schema = syndication_manager_schema
    allowed_content_types = (FEED_PORTAL_TYPE,)
    filter_content_types = True
    portal_type = meta_type = archetype_name = SYNDICATION_MANAGER_PORTAL_TYPE

    security.declareProtected(perms.ViewFeeds, 'getFeeds')
    def getFeeds(self):
        """See ISyndicationManager"""
        filter = {'portal_type': FEED_PORTAL_TYPE,}
        return self.contentValues(filter=filter)

    def getDefaultFeed(self):
        """See ISyndicationManager
        """
        dfid = self.getDefaultFeedId()
        if dfid:
            return getattr(self, dfid)
        return self.addDefaultFeed()
        
    def addDefaultFeed(self):
        """See ISyndicationManager
        """
        id = self.genId()
        self.invokeFactory(FEED_PORTAL_TYPE, id)
        self.setDefaultFeedId(id)
        feed = getattr(self, id)
        feed.addReferencedFeedSource(self.getContent())
        return feed

    def getFeedIds(self):
        return self.objectIds()

    def genId(self):
        while 1:
            id = str( random.randint(0, 99999) )
            if getattr(self, id, None) is None:
                break
        return id


registerType(SyndicationManager, PROJECTNAME)
